@@ -1,22 +1,22 @@
 BBreg <- function(formula,m,data=list(),maxiter=100){
-  
+
   if (sum(as.integer(m)==m)==length(m)){
   } else{
     stop("m must be integer")
   }
-  
+
   if (min(m)<=0){
     stop("m must be positive")
   }
-  
+
   if (maxiter!=as.integer(maxiter)){
     stop("maxiter must be integer")
   }
-  
+
   if(maxiter<=0){
     stop("maxiter must be positive")
   }
-  
+
   #Get the response, covariates and model matrices
   mf <- model.frame(formula=formula, data=data)
   X <- model.matrix(attr(mf, "terms"), data=mf)
@@ -35,21 +35,21 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
       balanced <- "no"
     }
   }
-  
+
   if (sum(as.integer(y)==y)==length(y)){
   } else {
     stop("y must be integer")
   }
-  
+
   if ((length(m)==1) | (length(m)==length(y))){
   } else{
     stop("m must be a number, or a vector of the length of y")
   }
-  
+
   if (max(y-m)>0 | min(y) < 0){
     stop("y must be bounded between 0 and m")
   }
-  
+
   # We calculate which y are 0 or m.
   t0 <- which(y==0)
   if (length(t0!=0)){
@@ -63,7 +63,7 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
   }else{
     ym <- y
   }
-  
+
   #BIG LOOP
   # Initial values
   iter <- 0
@@ -71,12 +71,12 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
   psi <- log(phi)
   oldphi <- 10000
   beta <- c(1,rep(0,dim(X)[2]-1))
-  
+
   while (abs(phi-oldphi)>0.001){
-    
+
     oldphi <- phi
     oldbeta <- rep(Inf,dim(X)[2])
-    
+
     #Beta LOOP
     beta.iter <- 0
     while (max(abs(beta-oldbeta))>0.001){
@@ -90,10 +90,10 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
       if(length(p0)!=0){
         p[p0] <- 0.00001
       }
-      
+
       # Compute S
       s <- p*(1-p)
-      
+
       # Compute u and v
       u <- NULL
       v <- NULL
@@ -123,18 +123,18 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
         u <- c(u,u1-u2)
         v <- c(v,v1+v2)
       }
-      
+
       # Computing y
       sv <- s*v
       Y <- X%*%beta+(1/sv)*u
-      
+
       # Updating beta
       XSV <- sweep(X,MARGIN=1,s*sqrt(v),'*')
       H <- solve(crossprod(XSV))
-      beta <- H%*%t(X)%*%as.vector(s*v*s*Y)
-      
+      beta <- tcrossprod(H,X)%*%as.vector(s*v*s*Y)
+
       beta.iter <- beta.iter+1
-      
+
       if (beta.iter>maxiter){
         print("The maximum number of iterations was has been reached, the method has not converged")
         conv <- "no"
@@ -142,9 +142,9 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
         return(out)
       }
     }
-    
+
     conv <- "yes"
-    
+
     # Once we reach convergence, we have to update phi using the profile likelihood
     if (length(t0)!=0){
       p0 <- p[-t0]
@@ -160,13 +160,13 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
       pm <- p
       mm <- m.
     }
-    
+
     # We define the profile likelihood of log(phi)
-    Lpsi <- function(psi){ 
+    Lpsi <- function(psi){
       Lp1 <- 0
       Lp2 <- 0
       Lp3 <- 0
-      
+
       for (j in 1:length(y0)){
         for (k in 0:(y0[j]-1)){
           Lp1 <- Lp1+k*exp(psi)/(p0[j]+k*exp(psi))
@@ -177,7 +177,7 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
           Lp2 <- Lp2+k*exp(psi)/(1-pm[j]+k*exp(psi))
         }
       }
-      for (j in 1:n){ 
+      for (j in 1:n){
         for (k in 0:(m.[j]-1)){
           Lp3 <- Lp3+k*exp(psi)/(1+k*exp(psi))
         }
@@ -189,10 +189,10 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
     psi.mle <- uniroot(Lpsi,lower=-20,upper=4,maxiter=maxiter,extendInt = "yes")
     psi <- psi.mle$root
     phi <- exp(psi)
-    
+
     #Number of iterations
     iter <- iter +1
-    
+
     if (iter>maxiter){
       print("The maximum number of iterations was has been reached, the method has not converged")
       conv <- "no"
@@ -200,7 +200,7 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
       return(out)
     }
   }
-  
+
   #VARIANCES ESTIMATION
   #beta
   vcov.b <- H
@@ -212,60 +212,60 @@ BBreg <- function(formula,m,data=list(),maxiter=100){
   }
   #log(phi)=psi
   psi.var <- -1/grad(Lpsi,psi)
-  
-  
+
+
   #Fitted value and residuals
   fitted.values <- p
 
   #DEVIANCE
-    dev1 <- dev1.null <- 0
-    dev2 <- dev2.null <- 0
-    dev3 <- dev3.null <- 0
-    
-    e <- sum(y)/sum(m.)
-    
-    for (j in 1:length(y0)){
-      for (k in 0:(y0[j]-1)){
-        dev1 <- dev1+log(p0[j]+k*phi)
-        dev1.null <- dev1.null+log(e+k*phi)
-      }
+  dev1 <- dev1.null <- 0
+  dev2 <- dev2.null <- 0
+  dev3 <- dev3.null <- 0
+
+  e <- sum(y)/sum(m.)
+
+  for (j in 1:length(y0)){
+    for (k in 0:(y0[j]-1)){
+      dev1 <- dev1+log(p0[j]+k*phi)
+      dev1.null <- dev1.null+log(e+k*phi)
     }
-    for (j in 1:length(ym)){
-      for (k in 0:(mm[j]-ym[j]-1)){
-        dev2 <- dev2+log(1-pm[j]+k*phi)
-        dev2.null <- dev2.null+log(1-e+k*phi)
-      }
+  }
+  for (j in 1:length(ym)){
+    for (k in 0:(mm[j]-ym[j]-1)){
+      dev2 <- dev2+log(1-pm[j]+k*phi)
+      dev2.null <- dev2.null+log(1-e+k*phi)
     }
-    for (j in 1:n){ 
-      for (k in 0:(m.[j]-1)){
-        dev3 <- dev3+log(1+k*phi)
-        dev3.null <- dev3.null+log(1+k*phi)
-      }
+  }
+  for (j in 1:n){
+    for (k in 0:(m.[j]-1)){
+      dev3 <- dev3+log(1+k*phi)
+      dev3.null <- dev3.null+log(1+k*phi)
     }
-    dev <- dev1+dev2-dev3
-    dev.null <- dev1.null+dev2.null-dev3.null
+  }
+  dev <- dev1+dev2-dev3
+  dev.null <- dev1.null+dev2.null-dev3.null
   #The deviance
   deviance <- as.numeric(2*(-sum(lgamma(m.+1)-(lgamma(y+1)+lgamma(m.-y+1)))-dev))
   deviance.null <- as.numeric(2*(-sum(lgamma(m.+1)-(lgamma(y+1)+lgamma(m.-y+1)))-dev.null))
-  
+
   #Degrees of freedom
   df <- n-length(beta)-1
   df.null <- n-1
-  
+
   #The output
   out <- list(coefficients=coef.b,vcov=vcov.b,
               phi=phi,psi=psi,psi.var=psi.var,
               conv=conv,fitted.values=fitted.values,
               deviance=deviance,df=df,null.deviance=deviance.null,null.df=df.null,
               iter=iter,X=X,y=y,m=m,balanced=balanced,nObs=n)
-  
+
   class(out) <- "BBreg"
-  
+
   out$call <- match.call()
   out$formula <- formula
-  
+
   out
-  
+
 }
 
 
@@ -290,14 +290,14 @@ summary.BBreg <- function(object,...){
   beta.tval <- object$coefficients/beta.se
   beta.TAB <- cbind(object$coefficients,beta.se,beta.tval,2*pt(-abs(beta.tval),df=object$df))
   colnames(beta.TAB) <- c("Estimate","StdErr","t.value","p.value")
-  
+
   coefficients.psi=cbind(object$psi,sqrt(object$psi.var))
   colnames(coefficients.psi) <- c("Estimate","StdErr")
   rownames(coefficients.psi) <- c("log(phi)")
-  
+
   Chi <- object$null.deviance-object$deviance
   Chi.p.value <- 1-pchisq(Chi,object$null.df-object$df)
-  
+
   out <- list(call=object$call,coefficients=beta.TAB,
               psi.table=coefficients.psi,
               deviance=object$deviance,df=object$df,null.deviance=object$null.deviance,null.df=object$null.df,
@@ -306,7 +306,7 @@ summary.BBreg <- function(object,...){
               X=object$X,y=object$y,Z=object$Z,nObs=object$nObs,
               m=object$m,balanced=object$balanced,
               conv=object$conv)
-  
+
   class(out) <- "summary.BBreg"
   out
 }
@@ -333,4 +333,4 @@ print.summary.BBreg <- function(x,...){
     cat("\nNo balanced data.")
   }
   cat("\nNumber of iterations:", x$iter,"\n")
-}
\ No newline at end of file
+}
